@@ -242,8 +242,14 @@ CCSSetting *
 ccsFindSetting (CCSPlugin * plugin, const char *name,
 		Bool isScreen, unsigned int screenNum)
 {
+    if (!plugin)
+	return NULL;
+
     PLUGIN_PRIV (plugin);
 
+    if (!name)
+	name = "";
+
     if (!pPrivate->loaded)
 	ccsLoadPluginSettings (plugin);
 
@@ -403,6 +409,11 @@ ccsFreePlugin (CCSPlugin * p)
     if (pPrivate->xmlPath)
 	free (pPrivate->xmlPath);
 
+#ifdef USE_PROTOBUF
+    if (pPrivate->pbFilePath)
+	free (pPrivate->pbFilePath);
+#endif
+
     free (pPrivate);
     free (p);
 }
@@ -615,6 +626,7 @@ openBackend (char *backend)
 Bool
 ccsSetBackend (CCSContext * context, char *name)
 {
+    Bool	 fallbackMode = FALSE;
     CONTEXT_PRIV (context);
 
     if (cPrivate->backend)
@@ -635,6 +647,7 @@ ccsSetBackend (CCSContext * context, char *name)
     void *dlhand = openBackend (name);
     if (!dlhand)
     {
+	fallbackMode = TRUE;
 	name = "ini";
 	dlhand = openBackend (name);
     }
@@ -669,7 +682,8 @@ ccsSetBackend (CCSContext * context, char *name)
 	cPrivate->backend->vTable->backendInit (context);
 
     ccsDisableFileWatch (cPrivate->configWatchId);
-    ccsWriteConfig (OptionBackend, name);
+    if (!fallbackMode)
+	ccsWriteConfig (OptionBackend, name);
     ccsEnableFileWatch (cPrivate->configWatchId);
 
     return TRUE;
@@ -1504,13 +1518,20 @@ ccsGetSortedPluginStringList (CCSContext * context)
 	ap = ccsPluginListRemove (ap, p, FALSE);
 
     int len = ccsPluginListLength (ap);
-    
+    if (len == 0)
+    {
+	ccsStringListFree (rv, TRUE);
+	return NULL;
+    }
     int i, j;
     /* TODO: conflict handling */
 
     PluginSortHelper *plugins = calloc (1, len * sizeof (PluginSortHelper));
     if (!plugins)
+    {
+	ccsStringListFree (rv, TRUE);
 	return NULL;
+    }
 
     for (i = 0, list = ap; i < len; i++, list = list->next)
     {
@@ -2625,7 +2646,10 @@ ccsImportFromFile (CCSContext *context,
 
 		    if (ccsIniGetString (importFile, plugin->name,
 					 keyName, &value))
-			ccsSetString (setting, value);
+		    {
+		    	ccsSetString (setting, value);
+		    	free (value);
+		    }
 		}
 		break;
 	    case TypeKey:
@@ -2678,7 +2702,10 @@ ccsImportFromFile (CCSContext *context,
 		    char *value;
 		    if (ccsIniGetString (importFile, plugin->name,
 					 keyName, &value))
-			ccsSetMatch (setting, value);
+		    {
+		    	ccsSetMatch (setting, value);
+		    	free (value);
+		    }
 		}
 		break;
 	    case TypeList:
